@@ -14,6 +14,8 @@
 
 # -*- coding: utf-8 -*-
 
+from unittest.mock import patch
+
 import numpy as np
 import pytest
 import tensorflow as tf
@@ -166,6 +168,25 @@ def test_sumkl_equals_batchkl(shared_k, diag):
     assert_almost_equal(kl_sum, kl_batch)
 
 
+@patch("tensorflow.__version__", "2.1.0")
+def test_sumkl_equals_batchkl_shared_k_not_diag_mocked_tf21():
+    """
+    Version of test_sumkl_equals_batchkl with shared_k=True and diag=False
+    that tests the TensorFlow < 2.2 workaround with tiling still works.
+    """
+    kl_batch = gauss_kl(Datum.mu, Datum.sqrt, Datum.K)
+    kl_sum = []
+    for n in range(Datum.N):
+        q_mu_n = Datum.mu[:, n][:, None]  # [M, 1]
+        q_sqrt_n = Datum.sqrt[n, :, :][None, :, :]  # [1, M, M] or [M, 1]
+        K_n = Datum.K  # [1, M, M] or [M, M]
+        kl_n = gauss_kl(q_mu_n, q_sqrt_n, K=K_n)
+        kl_sum.append(kl_n)
+
+    kl_sum = tf.reduce_sum(kl_sum)
+    assert_almost_equal(kl_sum, kl_batch)
+
+
 @pytest.mark.parametrize("dim", [0, 1])
 @pytest.mark.parametrize("white", [True, False])
 def test_oned(white, dim):
